#!/usr/bin/python3

import timeit

setup = """
from base64 import b32encode, b64encode
from _filestore import fast_b32encode

from filestore import check_id, B32ALPHABET
from filestore.protocols import Protocol, OldProtocol, MiB

digest = b'N' * 30
_id = 'GTUK6VPCITPAUSG3FLVNYK7IXEPXWDKGQ4T2X4IUOYXDE232'

new = Protocol(8 * MiB, 240)
old = OldProtocol(8 * MiB, 240)

leaf_index = 17
leaf_data = b'S' * 1776

file_size = 31415
leaf_hashes = b'D' * 30
"""

N = 50 * 1000

def run(statement):
    t = timeit.Timer(statement, setup)
    elapsed = t.timeit(N)
    rate = int(N / elapsed)
    print('{:>10,}: {}'.format(rate, statement))


print('Executions per second:')

run('b32encode(digest)')
run('fast_b32encode(digest)')
run("fast_b32encode(digest).decode('utf-8')")

run('b64encode(digest)')
run("b64encode(digest).decode('utf-8')")
run("b64encode(digest).decode('ascii')")

run('check_id(_id)')
run('set(_id).issubset(B32ALPHABET)')
run('B32ALPHABET.issuperset(_id)')

run("old._hash_leaf(leaf_index, leaf_data, b'')")
run("new._hash_leaf(leaf_index, leaf_data, b'')")

run('old._hash_root(file_size, leaf_hashes)')
run('new._hash_root(file_size, leaf_hashes)')
run('new.hash_root(file_size, leaf_hashes)')

run('str(4294967295)')
run("str(4294967295).encode('utf-8')")
