#!/usr/bin/env python3

# filestore: dmedia hashing protocol and file layout
# Copyright (C) 2011 Novacut Inc
#
# This file is part of `filestore`.
#
# `filestore` is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# `filestore` is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with `filestore`.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#   Jason Gerard DeRose <jderose@novacut.com>

import sys
from threading import Thread
import multiprocessing
from queue import Queue
from math import log, floor
import os
import time

from benchmark import units_base10, Timer
from filestore import hash_leaf, LEAF_SIZE, Leaf


def hasher_worker(q):
    while True:
        leaf = q.get()
        if leaf is None:
            break
        hash_leaf(leaf.index, leaf.data)


def hasher_thread():
    q = Queue(2)
    thread = Thread(target=hasher_worker, args=(q,))
    thread.daemon = True
    thread.start()
    return (q, thread)


src_fp = open(sys.argv[1], 'rb')
size = os.fstat(src_fp.fileno()).st_size


# Okay, here we go!
# You'll (likely) get peak thoughouput with with cpu_count() - 1 hashing threads
# so that reader thread has a core to itself
for count in range(1, multiprocessing.cpu_count() + 1):
    src_fp.seek(0)
    timer = Timer()
    queues = []
    threads = []
    for t in range(count):
        (q, thread) = hasher_thread()
        queues.append(q)
        threads.append(thread)

    i = 0
    while True:
        data = src_fp.read(LEAF_SIZE)
        if not data:
            break
        leaf = Leaf(i, data)
        q = queues[i % len(queues)]
        q.put(leaf)
        i += 1

    for q in queues:
        q.put(None)
    for t in threads:
        t.join()

    print('')
    print('Hashing threads: {}'.format(count))
    print('Time: {}'.format(timer.done()))
    print(timer.rate(size))
